C     -*- fortran -*-
C     This file is autogenerated with f2py (version:2)
C     It contains Fortran 77 wrappers to fortran functions.

      subroutine f2pywrapquicksort (array, first, last, indices, f
     &2py_array_d0, f2py_indices_d0)
      integer first
      integer last
      integer f2py_array_d0
      integer f2py_indices_d0
      real(kind=8) array(f2py_array_d0)
      integer indices(f2py_indices_d0)
      interface
      subroutine quicksort(array,first,last,indices) 
          real(kind=8), dimension(:),intent(inout) :: array
          integer, intent(in) :: first
          integer, intent(in) :: last
          integer, dimension(:),intent(inout) :: indices
      end subroutine quicksort
      end interface
      call quicksort(array, first, last, indices)
      end


      subroutine f2pywrapsplit (array, low, high, mid, indices, f2
     &py_array_d0, f2py_indices_d0)
      integer low
      integer high
      integer mid
      integer f2py_array_d0
      integer f2py_indices_d0
      real(kind=8) array(f2py_array_d0)
      integer indices(f2py_indices_d0)
      interface
      subroutine split(array,low,high,mid,indices) 
          real(kind=8), dimension(:),intent(inout) :: array
          integer, intent(in) :: low
          integer, intent(in) :: high
          integer, intent(out) :: mid
          integer, dimension(:),intent(inout) :: indices
      end subroutine split
      end interface
      call split(array, low, high, mid, indices)
      end

