#define  defineTIN   definetin
#define  defineTIN_   definetin_
#define  _defineTIN  _definetin
#define  _defineTIN_   _definetin_

#define  defineGTIN   definegtin
#define  defineGTIN_   definegtin_
#define  _defineGTIN  _definegtin
#define  _defineGTIN_   _definegtin_

#define  fillPIT   fillpit
#define  fillPIT_   fillpit_
#define  _fillPIT  _fillpit
#define  _fillPIT_  _fillpit_

#define  ngbGlob   ngbglob
#define  ngbGlob_   ngbglob_
#define  _ngbGlob  _ngbglob
#define  _ngbGlob_  _ngbglob_

#define  MFDreceivers   mfdreceivers
#define  MFDreceivers_   mfdreceivers_
#define  _MFDreceivers  _mfdreceivers
#define  _MFDreceivers_  _mfdreceivers_

#define  setMaxNb  setmaxnb
#define  setMaxNb_   setmaxnb_
#define  _setMaxNb   _setmaxnb
#define  _setMaxNb_   _setmaxnb_

#define  setHillslopeCoeff  sethillslopecoeff
#define  setHillslopeCoeff_   sethillslopecoeff_
#define  _setHillslopeCoeff   _sethillslopecoeff
#define  _setHillslopeCoeff_   _sethillslopecoeff_

#define  setDiffusionCoeff  setdiffusioncoeff
#define  setDiffusionCoeff_   setdiffusioncoeff_
#define  _setDiffusionCoeff   _setdiffusioncoeff
#define  _setDiffusionCoeff_   _setdiffusioncoeff_
